View subclass:#GraphColumnView
	instanceVariableNames:'columns vLines listHolder vLinesHolder zoomYHolder oldMenuMessage
		windowSizeHolder windowSize gridColor showGrid fgColor bgColor
		vLinesColor zoomY scrollUpdatesOriginX graphOriginX
		graphOriginXHolder'
	classVariableNames:'DefaultBackgroundColor DefaultGridColor DefaultForegroundColor
		DefaultVLinesColor'
	poolDictionaries:''
	category:'Views-Graphs'
!

!GraphColumnView class methodsFor:'documentation'!

documentation
"
    The class describes the common interface supported by the 2D or 3D GraphColumnView.
    This Viewclasses provide a lot of functionality for showing and manipulating graphs
    described through to a GraphColumn description. Each change in a graph description
    immediately take affect in the garph view.


    [See also:]
        GraphColumn
        GraphColumnView2D
        GraphColumnView3D

    [Author:]
        Claus Atzkern
"

! !

!GraphColumnView class methodsFor:'defaults'!

defaultMenu
    "redefined by subclass: should return the default middle button menu
    "
    ^ nil
!

updateStyleCache
    "extract values from the styleSheet and cache them in class variables
    "
    DefaultForegroundColor := Color black.
    DefaultBackgroundColor := Color veryLightGray.
    DefaultGridColor       := Color lightGray.
    DefaultVLinesColor     := DefaultForegroundColor
"
 self updateStyleCache
"


! !

!GraphColumnView methodsFor:'accessing'!

columns
    "returns the list of column descriptions
    "
    ^ columns


!

columns:aList
    "set the list of columns descriptions
    "
    columns notNil ifTrue:[
        columns do:[:aCol| aCol removeDependent:self ]
    ].

    aList size ~~ 0 ifTrue:[
        columns := OrderedCollection new.

        aList do:[:aColumn| 
            aColumn addDependent:self.
            columns add:aColumn
        ]
    ] ifFalse:[
        columns := nil
    ].
    self recomputeWholeGraph
!

graphOriginX
    "returns the logical index X of the first visible row; this number is used for
     accessing Y values from the GraphColumn description. On default, the value
     is set to 1.
    "
    ^ graphOriginX


!

graphOriginX:aNumber
    "set the logical index X of the first visible row; this number is used for
     accessing Y values from the GraphColumn description. On default, the value
     is set to 1.
     Changing the number, a scroll left or right is triggered.
    "
    |newX state|

    aNumber isNumber ifTrue:[
        newX := aNumber isInteger ifTrue:[aNumber]
                                 ifFalse:[(aNumber asFloat) rounded].   "/ no fractions

        newX ~~ graphOriginX ifTrue:[
            state := scrollUpdatesOriginX.
            scrollUpdatesOriginX := true.
            self scroll:(graphOriginX - newX).
            scrollUpdatesOriginX := state.
        ]
    ]
!

scrollUpdatesOriginX
    "returns true, if the graphOriginX automatically is updated by
     any scroll action. The default is set to false.
    "
    ^ scrollUpdatesOriginX
!

scrollUpdatesOriginX:aBool
    "set to true if the graphOriginX automatically should be updated by any
     scroll action. Otherwise a scroll has no influnce to the current
     graphOriginX.
     The default is set to false.
    "
    scrollUpdatesOriginX := aBool
!

showDefaultMenu
    "returns true, if the middleButton menu is set to the default menu
     provided by the graph.
    "
    ^ self menuMessage == #defaultMenu
!

showDefaultMenu:aBool
    "enable or disable the default menu provided by the graph
    "
    |currMsg|

    currMsg := self menuMessage.

    aBool ifTrue:[
        oldMenuMessage := currMsg.
        self menuMessage:#defaultMenu
    ] ifFalse:[
        currMsg == #defaultMenu ifTrue:[
            self menuMessage:oldMenuMessage
        ]
    ].


! !

!GraphColumnView methodsFor:'accessing dimensions'!

windowSize
    "get the number of horizontal steps ( X )
    "
    ^ windowSize


!

windowSize:aValue
    "set the number of horizontal steps ( X )
    "
    |sz|

    sz := (self unsignedIntegerFrom:aValue onError:[101]) max:5.

    sz ~~ windowSize ifTrue:[
        windowSize := sz.
        self recomputeWholeGraph
    ]

!

zoomY
    "returns current y-zoom factor
    "
    ^ zoomY


!

zoomY:aValue
    "set the current y-zoom factor; if the argument is nil,
     the y-zoom is set to 1.
    "
    |zY|

    (zY := self floatFrom:aValue onError:[1]) <= 0 ifTrue:[
        zY := 1
    ].

    zY = zoomY ifFalse:[
        zoomY := zY.
        self invalidateGraph
    ]


! !

!GraphColumnView methodsFor:'accessing look'!

backgroundColor
    "returns the current background color of the graph
    "
    ^ bgColor


!

backgroundColor:aColor
    "set the background color of the graph
    "
    (aColor isColor and:[bgColor ~= aColor]) ifTrue:[
        shown ifTrue:[
            bgColor := aColor on:device.
            self colorChanged:#background.
        ] ifFalse:[
            bgColor := aColor
        ]
    ]
!

foregroundColor
    "returns the default foreground color used to draw graphs which
     has no foreground color specified.
    "
    ^ fgColor

!

foregroundColor:aColor
    "set the default foreground color used to draw graphs which
     has no foreground color specified.
    "
    (aColor isColor and:[fgColor ~= aColor]) ifTrue:[
        shown ifTrue:[
            fgColor := aColor on:device.
            self colorChanged:#foreground
        ] ifFalse:[
            fgColor := aColor
        ]
    ]

!

gridColor
    "returns the foreground color of the grid
    "
    ^ gridColor

!

gridColor:aColor
    "set the foreground color of the grid
    "
    (aColor isColor and:[gridColor ~= aColor]) ifTrue:[
        shown ifTrue:[
            gridColor := aColor on:device.

            showGrid ifTrue:[
                self colorChanged:#grid
            ]
        ] ifFalse:[
            gridColor := aColor
        ]
    ]

!

showGrid
    "returns true if the grid is enabled
    "
    ^ showGrid

!

showGrid:aBool
    "set the visibility state of the grid
    "
    |hasGrid|

    showGrid ~~ aBool ifTrue:[
        showGrid := aBool.

        shown ifTrue:[
            self invalidateGraph
        ]
    ]

!

vLinesColor
    "returns the foreground color used to draw vertical lines
    "
    ^ vLinesColor


!

vLinesColor:aColor
    "set the foreground color used to draw vertical lines
    "
    (aColor isColor and:[vLinesColor ~= aColor]) ifTrue:[
        shown ifTrue:[
            vLinesColor := aColor on:device.
            self colorChanged:#vLines
        ] ifFalse:[
            vLinesColor := aColor
        ]
    ]

! !

!GraphColumnView methodsFor:'accessing mvc'!

graphOriginXHolder
    "returns the valueHolder, which keeps the current graphOriginX (see: #graphOriginX:)
    "
    ^ graphOriginXHolder
!

graphOriginXHolder:aHolder
    "set the valueHolder, which keeps the current graphOriginX (see: #graphOriginX:)
    "
    graphOriginXHolder == aHolder ifFalse:[
        graphOriginXHolder notNil ifTrue:[
            graphOriginXHolder removeDependent:self
        ].
        (graphOriginXHolder := aHolder) notNil ifTrue:[
            graphOriginXHolder addDependent:self
        ].
    ].
    self graphOriginX:(graphOriginXHolder value)

!

listHolder
    "returns the valueHolder, which keeps the list of column descriptions (see: #column:)
    "
    ^ listHolder


!

listHolder:aHolder
    "set the valueHolder, which keeps the list of column descriptions (see: #column:)
    "
    listHolder == aHolder ifFalse:[
        listHolder notNil ifTrue:[
            listHolder removeDependent:self
        ].
        (listHolder := aHolder) notNil ifTrue:[
            listHolder addDependent:self
        ].
    ].
    self columns:(listHolder value)

!

model:aModel
    "set the valueHolder which holds the selection and maybe the list of columnms
    "
    (model respondsTo:#list) ifTrue:[
        (model list == listHolder) ifTrue:[
            self listHolder:nil
        ]
    ].
    super model:aModel.

    aModel notNil ifTrue:[
        (aModel respondsTo:#list) ifTrue:[
            self listHolder:model list
        ]
    ]

!

vLinesHolder
    "returns the valueHolder, which keeps the list of vLines (see: #vLines:)
    "
    ^ vLinesHolder


!

vLinesHolder:aHolder
    "set the valueHolder, which keeps the list of vLines (see: #vLines:)
    "
    vLinesHolder == aHolder ifFalse:[
        vLinesHolder notNil ifTrue:[
            vLinesHolder removeDependent:self
        ].
        (vLinesHolder := aHolder) notNil ifTrue:[
            vLinesHolder addDependent:self
        ].
    ].
    self vLines:(vLinesHolder value)

!

windowSizeHolder
    "returns the valueHolder, which keeps the windowSize (see: #windowSize:)
    "
    ^ windowSizeHolder

!

windowSizeHolder:aHolder
    "set the valueHolder, which keeps the windowSize (see: #windowSize:)
    "
    windowSizeHolder == aHolder ifFalse:[
        windowSizeHolder notNil ifTrue:[
            windowSizeHolder removeDependent:self
        ].
        (windowSizeHolder := aHolder) notNil ifTrue:[
            windowSizeHolder addDependent:self
        ].
    ].
    self windowSize:(windowSizeHolder value)

!

zoomYHolder
    "returns the valueHolder, which keeps the zoom Y factor (see: #zoomY:)
    "
    ^ zoomYHolder

!

zoomYHolder:aHolder
    "set the valueHolder, which keeps the zoom Y factor (see: #zoomY:)
    "
    zoomYHolder == aHolder ifFalse:[
        zoomYHolder notNil ifTrue:[
            zoomYHolder removeDependent:self
        ].
        (zoomYHolder := aHolder) notNil ifTrue:[
            zoomYHolder addDependent:self
        ]
    ].
    self zoomY:(zoomYHolder value).

! !

!GraphColumnView methodsFor:'accessing vLines'!

vLineAdd:aLineIndex
    "add vertical line index to end of list
    "
    ^ self vLineAdd:aLineIndex beforeIndex:(vLines size + 1)
!

vLineAdd:aLineIndex beforeIndex:anIndex
    "add a vertical line index before an index
    "
    |x|

    vLines add:aLineIndex beforeIndex:anIndex.

    (shown and:[aLineIndex > graphOriginX]) ifTrue:[
        (x := ((aLineIndex - 1) * self stepX) rounded) < width ifTrue:[
            self vLinesSizeChanged:#insert: atX:x.
        ]
    ].
    ^ aLineIndex
!

vLineAddAll:aCollection beforeIndex:anIndex
    "add a collection of vertical line indices before an index
    "
    aCollection size ~~ 0 ifTrue:[
        vLines size == 0 ifTrue:[
            self vLines:aCollection
        ] ifFalse:[
            vLines addAll:aCollection beforeIndex:anIndex.
            self recomputeWholeGraph.
        ]
    ]

!

vLineRemove:aLineIndex
    "remove a vertical line index
    "
    ^ self vLineRemoveIndex:(vLines identityIndexOf:aLineIndex)

!

vLineRemoveAll
    "remove all vertical line indices
    "
    self vLines:nil
!

vLineRemoveIndex:anIndex
    "remove the vertical line index at an index
    "
    |lineIndex x|

    lineIndex := vLines removeAtIndex:anIndex.

    (shown and:[lineIndex > graphOriginX]) ifTrue:[
        (x := ((lineIndex - 1) * self stepX) rounded) < width ifTrue:[
            self vLinesSizeChanged:#remove: atX:x.
        ]
    ].
    ^ lineIndex


!

vLines
    "returns list of vertical lines
    "
    ^ vLines


!

vLines:aListOfIndices
    "set list of vertical lines
    "
    aListOfIndices size == 0 ifTrue:[
        vLines isEmpty ifTrue:[
            ^ self
        ]
    ] ifFalse:[
        vLines := OrderedCollection new.
        aListOfIndices do:[:i| vLines add:i ]
    ].
    shown ifTrue:[
        self recomputeWholeGraph
    ]
! !

!GraphColumnView methodsFor:'adding & removing'!

add:aColumn
    "insert a column at end; returns the inserted column
    "
    ^ self add:aColumn beforeIndex:(1 + columns size)

!

add:aColumn afterIndex:anIndex
    "add a new column after an index; returns the inserted column
    "
    ^ self add:aColumn beforeIndex:(anIndex + 1)

!

add:aColumn beforeIndex:anIndex
    "add a column before an index; returns the inserted column
    "
    aColumn isNil ifTrue:[^ nil].

    columns isNil ifTrue:[
        self columns:(Array with:aColumn).
        ^ aColumn.
    ].
    columns add:aColumn beforeIndex:anIndex.
    aColumn addDependent:self.

    aColumn shown ifTrue:[
        self listSizeChanged:#insert: from:aColumn.
    ].

!

addAll:aCollection beforeIndex:anIndex
    "add a collection of columns before an index
    "
    aCollection size ~~ 0 ifTrue:[
        columns size == 0 ifTrue:[
            self columns:aCollection
        ] ifFalse:[
            columns addAll:aCollection beforeIndex:anIndex.
            self recomputeWholeGraph.
        ]
    ]

!

addFirst:aColumn
    "insert a column at start; returns the inserted column
    "
    ^ self add:aColumn beforeIndex:1

!

removeAll
    "remove all columns
    "
    self columns:nil

!

removeFirst
    "remove first column; returns the removed column
    "
    ^ self removeIndex:1


!

removeIndex:anIndex
    "remove column at an index; returns the removed column
    "
    |col|

    col := columns removeAtIndex:anIndex.
    col removeDependent:self.

    columns size == 0 ifTrue:[
        columns := nil
    ].
    col shown ifTrue:[
        self listSizeChanged:#remove: from:col
    ].
  ^ col


!

removeLast
    "remove last column; the removed column is returned
    "
    ^ self removeIndex:(columns size)

! !

!GraphColumnView methodsFor:'change & update'!

update:what with:aPara from:chgObj
    "catch and handle a change notification of any object
    "
    |list start size stop|

    chgObj == windowSizeHolder ifTrue:[
        ^ self windowSize:(windowSizeHolder value)
    ].

    chgObj == zoomYHolder ifTrue:[
        ^ self zoomY:(zoomYHolder value)
    ].

    chgObj == graphOriginXHolder ifTrue:[
        ^ self graphOriginX:(graphOriginXHolder value)
    ].

    chgObj == vLinesHolder ifTrue:[
        (what == #insert:) ifTrue:[
            self vLineAdd:(list at:aPara) beforeIndex:aPara
        ] ifFalse:[
            (what == #remove:) ifTrue:[
                self vLineRemoveIndex:aPara
            ] ifFalse:[
                (what == #insertCollection:) ifTrue:[
                    start := aPara first.
                    size  := aPara last.

                    size ~~ 0 ifTrue:[
                        size == 1 ifTrue:[
                            self vLineAdd:(list at:start) beforeIndex:start
                        ] ifFalse:[
                            stop := start + size - 1.
                            self vLineAddAll:(list copyFrom:start to:stop) beforeIndex:start
                        ]
                    ]
                ] ifFalse:[
                    self vLinesHolder:chgObj
                ]
            ]
        ].
        ^ self        
    ].



    chgObj == model ifTrue:[
        (what == #selectionIndex or:[what == #selection]) ifTrue:[
            ^ self
        ].
        what == #list ifTrue:[
            ^ self listHolder:model list
        ].
        model == listHolder ifFalse:[
            ^ self
        ].
    ].

    chgObj == listHolder ifTrue:[
        list := listHolder value.

        (what == #insert:) ifTrue:[
            self add:(list at:aPara) beforeIndex:aPara
        ] ifFalse:[
            (what == #remove:) ifTrue:[
                self removeIndex:aPara
            ] ifFalse:[
                (what == #insertCollection:) ifTrue:[
                    start := aPara first.
                    size  := aPara last.

                    size ~~ 0 ifTrue:[
                        size == 1 ifTrue:[
                            self add:(list at:start) beforeIndex:start
                        ] ifFalse:[
                            stop := start + size - 1.
                            self addAll:(list copyFrom:start to:stop) beforeIndex:start
                        ]
                    ]
                ] ifFalse:[
                    self listHolder:chgObj
                ]
            ]
        ].
        ^ self
    ].

    columns notNil ifTrue:[
        (columns includesIdentical:chgObj) ifTrue:[
            ^ self columnChanged:what with:aPara from:chgObj
        ]
    ].

    super update:what with:aPara from:chgObj

! !

!GraphColumnView methodsFor:'conversion'!

floatFrom:aValue onError:aBlock
    "converts something to a float, on error the result of the block is returned
    "
    ^ aValue isNumber ifTrue:[aValue asFloat] ifFalse:[aBlock value]
!

unsignedIntegerFrom:aValue onError:aBlock
    "converts something to an unsigned integer, on error the result of the block is returned
    "
    |v|

    aValue isNumber ifTrue:[
        v := aValue isInteger ifTrue:[aValue] ifFalse:[(aValue asFloat) rounded].       "/ no fractions

        v >= 0 ifTrue:[ ^ v ]
    ].
    ^ aBlock value
! !

!GraphColumnView methodsFor:'initialization'!

create
    "set color on device
    "
    super create.

    fgColor     := (fgColor     ? DefaultForegroundColor) on:device.
    bgColor     := (bgColor     ? DefaultBackgroundColor) on:device.
    gridColor   := (gridColor   ? DefaultGridColor)       on:device.
    vLinesColor := (vLinesColor ? DefaultVLinesColor)     on:device.

!

destroy
    "remove dependencies
    "
    super destroy.

    listHolder         removeDependent:self.
    vLinesHolder       removeDependent:self.
    windowSizeHolder   removeDependent:self.
    zoomYHolder        removeDependent:self.
    graphOriginXHolder removeDependent:self.

    columns notNil ifTrue:[
        columns do:[:aCol| aCol removeDependent:self ]
    ].

!

initialize
    "setup default values
    "
    super initialize.

    DefaultGridColor isNil ifTrue:[
        self class updateStyleCache
    ].

    vLines       := OrderedCollection new.
    windowSize   := 101.
    showGrid     := false.
    zoomY        := 1.
    graphOriginX := 1.
    scrollUpdatesOriginX := false.
! !

!GraphColumnView methodsFor:'menu & submenus'!

defaultMenu
    "returns the default middle button menu provided by the graph
    "
    |menu|

    menu := self class defaultMenu decodeAsLiteralArray.

    menu notNil ifTrue:[
        menu receiver:self
    ].
  ^ menu


!

doZoomY:aValue
    "triggered by the default menu to change the current zoom Y factor;
     on change, the corresponding model (zoomYHolder) is updated.
    "
    |old|

    old := self zoomY.
    self zoomY:aValue.

    self zoomYHolder notNil ifTrue:[
        zoomYHolder value:(self zoomY)
    ]
!

subMenuZoomY
    "returns the submenu to configure the zoom Y factor
    "
    ^ GraphColumn zoomMenuSelector:#doZoomY:


! !

!GraphColumnView methodsFor:'protocol'!

colorChanged:what
    "called if any color changed; the argument to the change notification
     specifies the color which has changed:

        #foreground     the foreground color 
        #background     the background color
        #grid           the color of the grid
        #vLines         the color of the vertical lines
    "
    self recomputeWholeGraph
!

columnChanged:what with:aPara from:aColumn
    "a column has changed one of its attributes; the arguments to the
     notification are passed by the column.
    "
    self recomputeWholeGraph


!

graphOriginXChanged:deltaX
    "scroll left or right n x-steps. A positive value scrolls to the right
     a negative value to the left.
    "
    self recomputeWholeGraph
!

invalidateGraph
    "called to redraw the graph
    "
    self recomputeWholeGraph
!

listSizeChanged:what from:aColumn
    "called if a column description is inserted (#insert:) or removed (#remove:)
     from the list of columns.
    "
    self recomputeWholeGraph
!

recomputeWholeGraph
    "called if the whole graph should be recomputed
    "
    self subclassResponsibility
!

vLinesSizeChanged:what atX:aPhysX
    "called if the list of vertical lines changed; a new lineIndex is inserted (#insert:)
     or removed (#remove:) from the list
    "
    self recomputeWholeGraph
! !

!GraphColumnView methodsFor:'queries'!

numberOfVisibleColumns
    "returns the number of visible Columns (shown is true)
    "
    |no|

    no := 0.

    columns notNil ifTrue:[
        columns do:[:aCol| aCol shown ifTrue:[no := no + 1]]
    ].
    ^ no
! !

!GraphColumnView methodsFor:'scrolling'!

halfWindowSizeLeft
    "scroll left half window size
    "
    self scrollLeft:(windowSize // 2)

!

halfWindowSizeRight
    "scroll right half window size
    "
    self scrollRight:(windowSize // 2)

!

scroll:nIndices
    "scroll left or right n x-steps. a positive value scrolls to the right
     a negative value to the left.
    "
    |max|

    nIndices ~~ 0 ifTrue:[
        scrollUpdatesOriginX ifTrue:[
            graphOriginX := graphOriginX - nIndices
        ].

        shown ifTrue:[
            max := 2 * (windowSize // 3).

            (nIndices abs) > max ifTrue:[
                self recomputeWholeGraph                "/ full redraw
            ] ifFalse:[
                self graphOriginXChanged:nIndices       "/ scroll
            ]
        ]
    ].
!

scrollLeft:nIndices
    "scroll n indices left
    "
    self scroll:(nIndices negated)

!

scrollRight:nIndices
    "scroll n indices right
    "
    self scroll:nIndices

!

windowSizeLeft
    "scroll left window size
    "
    self scrollLeft:windowSize

!

windowSizeRight
    "scroll right window size
    "
    self scrollRight:windowSize

! !

!GraphColumnView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/GraphColumnView.st,v 1.2 1998-02-09 10:31:27 ca Exp $'
! !
